@@ -1,6 +1,6 @@
 /* Test file for mpfr_cmp_d.
 
-Copyright 1999, 2001-2004, 2006-2017 Free Software Foundation, Inc.
+Copyright 1999, 2001-2004, 2006-2023 Free Software Foundation, Inc.
 Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
@@ -17,10 +17,11 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
 #include "mpfr-test.h"
+#include "ieee_floats.h"
 
 int
 main (void)
@@ -86,7 +87,7 @@ main (void)
     int c;
 
     mpfr_clear_flags ();
-    c = mpfr_cmp_d (x, DBL_NAN);
+    c = mpfr_cmp_d (x, MPFR_DBL_NAN);
     if (c != 0 || __gmpfr_flags != MPFR_FLAGS_ERANGE)
       {
         printf ("ERROR for NAN (1)\n");
